# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class CodingAttributionType(enum.StrEnum):
    CANDID = "CANDID"
    CUSTOMER = "CUSTOMER"
    TCN = "TCN"
    PJF = "PJF"
    _UNKNOWN = "__CODINGATTRIBUTIONTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "CodingAttributionType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        candid: typing.Callable[[], T_Result],
        customer: typing.Callable[[], T_Result],
        tcn: typing.Callable[[], T_Result],
        pjf: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is CodingAttributionType.CANDID:
            return candid()
        if self is CodingAttributionType.CUSTOMER:
            return customer()
        if self is CodingAttributionType.TCN:
            return tcn()
        if self is CodingAttributionType.PJF:
            return pjf()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class InvoiceStatus(enum.StrEnum):
    DRAFT = "DRAFT"
    OPEN = "OPEN"
    PAID = "PAID"
    VOID = "VOID"
    UNCOLLECTIBLE = "UNCOLLECTIBLE"
    HELD = "HELD"
    _UNKNOWN = "__INVOICESTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "InvoiceStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        draft: typing.Callable[[], T_Result],
        open: typing.Callable[[], T_Result],
        paid: typing.Callable[[], T_Result],
        void: typing.Callable[[], T_Result],
        uncollectible: typing.Callable[[], T_Result],
        held: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is InvoiceStatus.DRAFT:
            return draft()
        if self is InvoiceStatus.OPEN:
            return open()
        if self is InvoiceStatus.PAID:
            return paid()
        if self is InvoiceStatus.VOID:
            return void()
        if self is InvoiceStatus.UNCOLLECTIBLE:
            return uncollectible()
        if self is InvoiceStatus.HELD:
            return held()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.external_provider import ExternalProvider
from .....common.types.patient_service_facility import PatientServiceFacility
from .....common.types.period import Period
from .referral_unit import ReferralUnit


class Referral(UniversalBaseModel):
    provider: ExternalProvider
    referral_number: str
    period: typing.Optional[Period] = None
    notes: typing.Optional[str] = None
    service_facility: typing.Optional[PatientServiceFacility] = pydantic.Field(alias="serviceFacility", default=None)
    units: typing.Optional[ReferralUnit] = None
    quantity: typing.Optional[int] = None
    cpt_codes: typing.Optional[typing.List[str]] = pydantic.Field(alias="cptCodes", default=None)
    apply_for_all_cpt_codes: typing.Optional[bool] = pydantic.Field(alias="applyForAllCptCodes", default=None)

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....diagnoses.types.diagnosis_id import DiagnosisId
from .....encounters_universal.types.universal_encounter_update_base import UniversalEncounterUpdateBase
from .vitals_update import VitalsUpdate


class EncounterUpdate(UniversalEncounterUpdateBase):
    vitals: typing.Optional[VitalsUpdate] = pydantic.Field(default=None)
    """
    If a vitals entity already exists for the encounter, then all values will be updated to the provided values.
    Otherwise, a new vitals object will be created for the encounter.
    """

    diagnosis_ids: typing.Optional[typing.List[DiagnosisId]] = pydantic.Field(default=None)
    """
    Ideally, this field should contain no more than 12 diagnoses. However, more diagnoses
    may be submitted at this time, and coders will later prioritize the 12 that will be
    submitted to the payor.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.claim_id import ClaimId
from .patient_payment_info import PatientPaymentInfo
from .service_line_itemization import ServiceLineItemization


class InvoiceItemizationResponse(UniversalBaseModel):
    claim_id: ClaimId = pydantic.Field()
    """
    The ID of the claim associated with the invoice itemization.
    """

    patient_balance_cents: int = pydantic.Field()
    """
    The total patient balance in cents for the claim. Negative values indicate a credit balance.
    """

    claim_level_patient_payments: PatientPaymentInfo
    service_line_itemization: typing.List[ServiceLineItemization] = pydantic.Field()
    """
    A list of service line itemizations for the claim.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .diagnosis import Diagnosis
    from .diagnosis_create import DiagnosisCreate
    from .diagnosis_create_optional import DiagnosisCreateOptional
    from .diagnosis_id import DiagnosisId
    from .diagnosis_not_found_error import DiagnosisNotFoundError
    from .diagnosis_type_code import DiagnosisTypeCode
    from .disallow_multiple_primary_diagnosis_error import DisallowMultiplePrimaryDiagnosisError
    from .service_lines_must_have_at_least_one_diagnosis_error import ServiceLinesMustHaveAtLeastOneDiagnosisError
    from .standalone_diagnosis_create import StandaloneDiagnosisCreate
_dynamic_imports: typing.Dict[str, str] = {
    "Diagnosis": ".diagnosis",
    "DiagnosisCreate": ".diagnosis_create",
    "DiagnosisCreateOptional": ".diagnosis_create_optional",
    "DiagnosisId": ".diagnosis_id",
    "DiagnosisNotFoundError": ".diagnosis_not_found_error",
    "DiagnosisTypeCode": ".diagnosis_type_code",
    "DisallowMultiplePrimaryDiagnosisError": ".disallow_multiple_primary_diagnosis_error",
    "ServiceLinesMustHaveAtLeastOneDiagnosisError": ".service_lines_must_have_at_least_one_diagnosis_error",
    "StandaloneDiagnosisCreate": ".standalone_diagnosis_create",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Diagnosis",
    "DiagnosisCreate",
    "DiagnosisCreateOptional",
    "DiagnosisId",
    "DiagnosisNotFoundError",
    "DiagnosisTypeCode",
    "DisallowMultiplePrimaryDiagnosisError",
    "ServiceLinesMustHaveAtLeastOneDiagnosisError",
    "StandaloneDiagnosisCreate",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.decimal import Decimal
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.service_line_units import ServiceLineUnits
from .....encounter_providers.resources.v_2.types.ordering_provider_optional import OrderingProviderOptional
from .drug_identification_optional import DrugIdentificationOptional
from .test_result_optional import TestResultOptional


class ServiceLineCreateOptional(UniversalBaseModel):
    modifiers: typing.Optional[typing.List[ProcedureModifier]] = None
    has_epsdt_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-11 on the 837-P and Box 24H on the CMS-1500.
    If the value is true, the box will be populated with "Y". Otherwise, the box will not be populated.
    """

    has_family_planning_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-12 on the 837-P and Box 24I on the CMS-1500.
    If the value is true, the box will be populated with "Y". Otherwise, the box will not be populated.
    """

    procedure_code: typing.Optional[str] = None
    quantity: typing.Optional[Decimal] = pydantic.Field(default=None)
    """
    String representation of a Decimal that can be parsed by most libraries.
    For professional claims, a ServiceLine quantity cannot contain more than one digit of precision
    (Example: 1.1 is valid, 1.11 is not). For institutional claims, a ServiceLine quantity cannot contain
    more than three decimal digits of precision.
    """

    units: typing.Optional[ServiceLineUnits] = None
    charge_amount_cents: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total amount charged for this service line, factoring in quantity. If procedure_code is updated and this is not, the system will attempt to set it based on chargemasters entries and the service line’s quantity. For example, if a single unit has an entry of 100 cents and 2 units were rendered, the charge_amount_cents will be set to 200, if there is no chargemaster entry, it will default to the amount set in this field.
    """

    diagnosis_pointers: typing.Optional[typing.List[int]] = pydantic.Field(default=None)
    """
    Indices (zero-indexed) of all the diagnoses this service line references
    """

    drug_identification: typing.Optional[DrugIdentificationOptional] = None
    place_of_service_code: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    837p Loop2300, SV105. This enum is not used or required in 837i claims. If your organization does not intend to submit claims with a different place of service at the service line level, this field should not be populated. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A free-form description to clarify the related data elements and their content. Maps to SV1-01, C003-07 on a 837-P and SV2-02, C003-07 on a 837-I form.
    """

    date_of_service: typing.Optional[dt.date] = None
    end_date_of_service: typing.Optional[dt.date] = None
    ordering_provider: typing.Optional[OrderingProviderOptional] = pydantic.Field(default=None)
    """
    Required when the service or supply was ordered by a provider who is different than the rendering provider for this service line.
    If not required by this implementation guide, do not send.
    """

    test_results: typing.Optional[typing.List[TestResultOptional]] = pydantic.Field(default=None)
    """
    Contains a list of test results. Test result types may map to MEA-02 on the 837-P (ex: Hemoglobin, Hematocrit).
    This is unused by 837-i and ignored for institutional service lines.
    No more than 5 MEA-02 test results may be submitted per service line.
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    Maps to NTE02 loop 2400 on the EDI 837.
    """

    revenue_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    A 4 digit code that specifies facility department or type of service arrangement for institutional service line items (837i). This code is not required for professional claim billing (837p).
    """

    prior_authorization_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Prior authorization number for this service line. Maps to the appropriate REF segment on Loop 2400 of the EDI 837p. This is not used for institutional claims (EDI 837i).
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An external identifier for this service line. This is not used in the EDI 837, but can be used to identify the service line in external systems.
    This field should not contain PHI. Must be unique on a given claim.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class ContractSortField(enum.StrEnum):
    CREATED_AT = "created_at"
    PAYER_NAME = "payer_name"
    PAYER_ID = "payer_id"
    BILLING_PROVIDER_NAME = "billing_provider_name"
    EFFECTIVE_DATE = "effective_date"
    CONTRACT_STATUS = "contract_status"
    _UNKNOWN = "__CONTRACTSORTFIELD_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ContractSortField":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        created_at: typing.Callable[[], T_Result],
        payer_name: typing.Callable[[], T_Result],
        payer_id: typing.Callable[[], T_Result],
        billing_provider_name: typing.Callable[[], T_Result],
        effective_date: typing.Callable[[], T_Result],
        contract_status: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ContractSortField.CREATED_AT:
            return created_at()
        if self is ContractSortField.PAYER_NAME:
            return payer_name()
        if self is ContractSortField.PAYER_ID:
            return payer_id()
        if self is ContractSortField.BILLING_PROVIDER_NAME:
            return billing_provider_name()
        if self is ContractSortField.EFFECTIVE_DATE:
            return effective_date()
        if self is ContractSortField.CONTRACT_STATUS:
            return contract_status()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...commons.types.appointment_id import AppointmentId
from ...commons.types.claim_id import ClaimId
from ...commons.types.encounter_id import EncounterId
from ...commons.types.patient_external_id import PatientExternalId
from ...commons.types.provider_id import ProviderId
from ...commons.types.service_line_id import ServiceLineId


class AllocationTarget_ServiceLine(UniversalBaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    type: typing.Literal["service_line"] = "service_line"
    service_line_id: ServiceLineId
    claim_id: ClaimId
    encounter_id: EncounterId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AllocationTarget_Claim(UniversalBaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    type: typing.Literal["claim"] = "claim"
    claim_id: ClaimId
    encounter_id: EncounterId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AllocationTarget_BillingProviderId(UniversalBaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    type: typing.Literal["billing_provider_id"] = "billing_provider_id"
    billing_provider_id: ProviderId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AllocationTarget_Appointment(UniversalBaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    type: typing.Literal["appointment"] = "appointment"
    appointment_id: AppointmentId
    patient_external_id: PatientExternalId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AllocationTarget_Unattributed(UniversalBaseModel):
    """
    Allocation targets describe whether the portion of a payment is being applied toward a specific service line,
    claim, billing provider, or is unallocated.
    """

    type: typing.Literal["unattributed"] = "unattributed"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


AllocationTarget = typing_extensions.Annotated[
    typing.Union[
        AllocationTarget_ServiceLine,
        AllocationTarget_Claim,
        AllocationTarget_BillingProviderId,
        AllocationTarget_Appointment,
        AllocationTarget_Unattributed,
    ],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.api_error import ApiError
from ..types.updates_disabled_due_to_external_system_integration_error_message import (
    UpdatesDisabledDueToExternalSystemIntegrationErrorMessage,
)


class UpdatesDisabledDueToExternalSystemIntegrationError(ApiError):
    def __init__(
        self,
        body: UpdatesDisabledDueToExternalSystemIntegrationErrorMessage,
        headers: typing.Optional[typing.Dict[str, str]] = None,
    ):
        super().__init__(status_code=422, headers=headers, body=body)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.facility_type_code import FacilityTypeCode
from ....commons.types.network_type import NetworkType
from ....commons.types.page_token import PageToken
from ....commons.types.payer_plan_group_id import PayerPlanGroupId
from ....commons.types.procedure_modifier import ProcedureModifier
from ....commons.types.rate_id import RateId
from ....commons.types.service_line_id import ServiceLineId
from ....commons.types.state import State
from ....organization_providers.resources.v_2.types.license_type import LicenseType
from ....organization_providers.resources.v_2.types.organization_provider_id import OrganizationProviderId
from ....payers.resources.v_3.types.payer_uuid import PayerUuid
from .raw_client import AsyncRawV3Client, RawV3Client
from .types.dimension_name import DimensionName
from .types.dimensions_page import DimensionsPage
from .types.match_result import MatchResult
from .types.match_test_result import MatchTestResult
from .types.optional_dimensions import OptionalDimensions
from .types.payer_threshold import PayerThreshold
from .types.payer_thresholds_page import PayerThresholdsPage
from .types.rate import Rate
from .types.rate_upload import RateUpload
from .types.rates_page import RatesPage

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV3Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV3Client
        """
        return self._raw_client

    def get_match(
        self, service_line_id: ServiceLineId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[MatchResult]:
        """
        Gets the rate that matches a service line.  No result means no rate exists matching the service line's dimensions.

        Parameters
        ----------
        service_line_id : ServiceLineId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[MatchResult]

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.get_match(
            service_line_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get_match(service_line_id, request_options=request_options)
        return _response.data

    def test_match(
        self,
        service_line_id: ServiceLineId,
        rate_id: RateId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MatchTestResult:
        """
        Tests a service line against a rate to see if it matches.

        Parameters
        ----------
        service_line_id : ServiceLineId

        rate_id : RateId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MatchTestResult

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.test_match(
            service_line_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            rate_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.test_match(service_line_id, rate_id, request_options=request_options)
        return _response.data

    def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        active_date: typing.Optional[dt.date] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        organization_billing_provider_id: typing.Optional[OrganizationProviderId] = None,
        states: typing.Optional[typing.Union[State, typing.Sequence[State]]] = None,
        zip_codes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        license_types: typing.Optional[typing.Union[LicenseType, typing.Sequence[LicenseType]]] = None,
        facility_type_codes: typing.Optional[typing.Union[FacilityTypeCode, typing.Sequence[FacilityTypeCode]]] = None,
        network_types: typing.Optional[typing.Union[NetworkType, typing.Sequence[NetworkType]]] = None,
        payer_plan_group_ids: typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]] = None,
        cpt_code: typing.Optional[str] = None,
        modifiers: typing.Optional[typing.Union[ProcedureModifier, typing.Sequence[ProcedureModifier]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RatesPage:
        """
        Gets a list of dimensions with their rates. The rates returned will always be the most recent versions of those rates.

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Max number of dimensions returned. Defaults to 100. Max is 100.

        active_date : typing.Optional[dt.date]

        payer_uuid : typing.Optional[PayerUuid]

        organization_billing_provider_id : typing.Optional[OrganizationProviderId]

        states : typing.Optional[typing.Union[State, typing.Sequence[State]]]

        zip_codes : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        license_types : typing.Optional[typing.Union[LicenseType, typing.Sequence[LicenseType]]]

        facility_type_codes : typing.Optional[typing.Union[FacilityTypeCode, typing.Sequence[FacilityTypeCode]]]

        network_types : typing.Optional[typing.Union[NetworkType, typing.Sequence[NetworkType]]]

        payer_plan_group_ids : typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]]

        cpt_code : typing.Optional[str]

        modifiers : typing.Optional[typing.Union[ProcedureModifier, typing.Sequence[ProcedureModifier]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RatesPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.get_multi()
        """
        _response = self._raw_client.get_multi(
            page_token=page_token,
            limit=limit,
            active_date=active_date,
            payer_uuid=payer_uuid,
            organization_billing_provider_id=organization_billing_provider_id,
            states=states,
            zip_codes=zip_codes,
            license_types=license_types,
            facility_type_codes=facility_type_codes,
            network_types=network_types,
            payer_plan_group_ids=payer_plan_group_ids,
            cpt_code=cpt_code,
            modifiers=modifiers,
            request_options=request_options,
        )
        return _response.data

    def get_unique_values_for_dimension(
        self,
        *,
        pivot_dimension: DimensionName,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        organization_billing_provider_id: typing.Optional[OrganizationProviderId] = None,
        states: typing.Optional[typing.Union[State, typing.Sequence[State]]] = None,
        zip_codes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        license_types: typing.Optional[typing.Union[LicenseType, typing.Sequence[LicenseType]]] = None,
        facility_type_codes: typing.Optional[typing.Union[FacilityTypeCode, typing.Sequence[FacilityTypeCode]]] = None,
        network_types: typing.Optional[typing.Union[NetworkType, typing.Sequence[NetworkType]]] = None,
        payer_plan_group_ids: typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]] = None,
        cpt_code: typing.Optional[str] = None,
        modifiers: typing.Optional[typing.Union[ProcedureModifier, typing.Sequence[ProcedureModifier]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DimensionsPage:
        """
        Gets unique values for a dimension based on other selection criteria. The response is a list of dimensions with your criteria and the unique values populated. This API is useful for driving pivots on dimension values.

        Parameters
        ----------
        pivot_dimension : DimensionName
            The name of the dimension to fetch unique values for.

        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Max number of values returned. Defaults to 1000. Max is 1000.

        payer_uuid : typing.Optional[PayerUuid]

        organization_billing_provider_id : typing.Optional[OrganizationProviderId]

        states : typing.Optional[typing.Union[State, typing.Sequence[State]]]

        zip_codes : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        license_types : typing.Optional[typing.Union[LicenseType, typing.Sequence[LicenseType]]]

        facility_type_codes : typing.Optional[typing.Union[FacilityTypeCode, typing.Sequence[FacilityTypeCode]]]

        network_types : typing.Optional[typing.Union[NetworkType, typing.Sequence[NetworkType]]]

        payer_plan_group_ids : typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]]

        cpt_code : typing.Optional[str]

        modifiers : typing.Optional[typing.Union[ProcedureModifier, typing.Sequence[ProcedureModifier]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DimensionsPage

        Examples
        --------
        from candid import CandidApiClient
        from candid.resources.fee_schedules.resources.v_3 import DimensionName

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.get_unique_values_for_dimension(
            pivot_dimension=DimensionName.PAYER_UUID,
        )
        """
        _response = self._raw_client.get_unique_values_for_dimension(
            pivot_dimension=pivot_dimension,
            page_token=page_token,
            limit=limit,
            payer_uuid=payer_uuid,
            organization_billing_provider_id=organization_billing_provider_id,
            states=states,
            zip_codes=zip_codes,
            license_types=license_types,
            facility_type_codes=facility_type_codes,
            network_types=network_types,
            payer_plan_group_ids=payer_plan_group_ids,
            cpt_code=cpt_code,
            modifiers=modifiers,
            request_options=request_options,
        )
        return _response.data

    def get_rate_history(
        self, rate_id: RateId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Rate]:
        """
        Gets every version of a rate.

        Parameters
        ----------
        rate_id : RateId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Rate]

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.get_rate_history(
            rate_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get_rate_history(rate_id, request_options=request_options)
        return _response.data

    def upload_fee_schedule(
        self,
        *,
        dry_run: bool,
        rates: typing.Sequence[RateUpload],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Rate]:
        """
        Uploads a new fee schedule.

        Each rate may either be totally new as qualified by it's dimensions or a new version for an existing rate.

        If adding a new version to an existing rate, the rate must be posted with the next version number (previous version + 1) or a EntityConflictError will
        be returned.

        Use the dry run flag to discover already existing rates and to run validations.  If validations for any rate fail, no rates will
        be saved to the system.

        Parameters
        ----------
        dry_run : bool

        rates : typing.Sequence[RateUpload]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Rate]

        Examples
        --------
        import datetime
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            NetworkType,
            ProcedureModifier,
            State,
        )
        from candid.resources.fee_schedules.resources.v_3 import (
            Dimensions,
            RateEntry,
            RateUpload_NewRate,
        )
        from candid.resources.organization_providers.resources.v_2 import LicenseType

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.upload_fee_schedule(
            dry_run=True,
            rates=[
                RateUpload_NewRate(
                    dimensions=Dimensions(
                        payer_uuid=uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        organization_billing_provider_id=uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        states=[State.AA],
                        zip_codes={"zip_codes"},
                        license_types=[LicenseType.MD],
                        facility_type_codes=[FacilityTypeCode.PHARMACY],
                        network_types=[NetworkType.PPO],
                        payer_plan_group_ids=[
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            )
                        ],
                        cpt_code="cpt_code",
                        modifiers=[ProcedureModifier.AV],
                    ),
                    entries=[
                        RateEntry(
                            start_date=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            rate_cents=1,
                            is_deactivated=True,
                        ),
                        RateEntry(
                            start_date=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            rate_cents=1,
                            is_deactivated=True,
                        ),
                    ],
                ),
                RateUpload_NewRate(
                    dimensions=Dimensions(
                        payer_uuid=uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        organization_billing_provider_id=uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        states=[State.AA],
                        zip_codes={"zip_codes"},
                        license_types=[LicenseType.MD],
                        facility_type_codes=[FacilityTypeCode.PHARMACY],
                        network_types=[NetworkType.PPO],
                        payer_plan_group_ids=[
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            )
                        ],
                        cpt_code="cpt_code",
                        modifiers=[ProcedureModifier.AV],
                    ),
                    entries=[
                        RateEntry(
                            start_date=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            rate_cents=1,
                            is_deactivated=True,
                        ),
                        RateEntry(
                            start_date=datetime.date.fromisoformat(
                                "2023-01-15",
                            ),
                            rate_cents=1,
                            is_deactivated=True,
                        ),
                    ],
                ),
            ],
        )
        """
        _response = self._raw_client.upload_fee_schedule(dry_run=dry_run, rates=rates, request_options=request_options)
        return _response.data

    def delete_rate(
        self, rate_id: RateId, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Soft deletes a rate from the system.  Only the most recent version of a rate can be deleted.

        Parameters
        ----------
        rate_id : RateId

        version : int

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.delete_rate(
            rate_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            version=1,
        )
        """
        _response = self._raw_client.delete_rate(rate_id, version, request_options=request_options)
        return _response.data

    def get_payer_thresholds_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerThreshold:
        """
        Gets the default payer threshold

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerThreshold

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.get_payer_thresholds_default()
        """
        _response = self._raw_client.get_payer_thresholds_default(request_options=request_options)
        return _response.data

    def get_payer_thresholds(
        self,
        *,
        payer_uuids: typing.Union[PayerUuid, typing.Sequence[PayerUuid]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayerThresholdsPage:
        """
        Gets a list of payers and thresholds by their uuids

        Parameters
        ----------
        payer_uuids : typing.Union[PayerUuid, typing.Sequence[PayerUuid]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerThresholdsPage

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.get_payer_thresholds(
            payer_uuids=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get_payer_thresholds(payer_uuids=payer_uuids, request_options=request_options)
        return _response.data

    def set_payer_threshold(
        self, payer_uuid: PayerUuid, *, request: PayerThreshold, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerThreshold:
        """
        Sets the threshold information for a payer

        Parameters
        ----------
        payer_uuid : PayerUuid

        request : PayerThreshold

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerThreshold

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.fee_schedules.resources.v_3 import PayerThreshold

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.set_payer_threshold(
            payer_uuid=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=PayerThreshold(
                disable_paid_incorrectly=True,
            ),
        )
        """
        _response = self._raw_client.set_payer_threshold(payer_uuid, request=request, request_options=request_options)
        return _response.data

    def hard_delete_rates(
        self, *, request: OptionalDimensions, request_options: typing.Optional[RequestOptions] = None
    ) -> int:
        """
        Hard deletes rates from the system that match the provided dimensions.  This is a destructive operation and cannot be undone.  If an empty dimensions object is provided, all rates will be hard deleted.  The maximum number of rates this API will delete at a time is 10000.  Returns the number of rates deleted and if that number is the maximum, the caller should call this API again to continue deleting rates.

        Parameters
        ----------
        request : OptionalDimensions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        int

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            NetworkType,
            ProcedureModifier,
            State,
        )
        from candid.resources.fee_schedules.resources.v_3 import OptionalDimensions
        from candid.resources.organization_providers.resources.v_2 import LicenseType

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.hard_delete_rates(
            request=OptionalDimensions(
                states=[State.AA],
                zip_codes={"zip_codes"},
                license_types=[LicenseType.MD],
                facility_type_codes=[FacilityTypeCode.PHARMACY],
                network_types=[NetworkType.PPO],
                payer_plan_group_ids=[
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    )
                ],
                modifiers=[ProcedureModifier.AV],
            ),
        )
        """
        _response = self._raw_client.hard_delete_rates(request=request, request_options=request_options)
        return _response.data

    def hard_delete_rates_by_ids(
        self, *, rate_ids: typing.Sequence[RateId], request_options: typing.Optional[RequestOptions] = None
    ) -> int:
        """
        Hard deletes specific rates from the system by their IDs. This is a destructive operation and cannot be undone. Limited to 100 rate IDs maximum per request. For bulk deletion of more than 100 rates, use the hard_delete_rates endpoint with dimension filters. Returns the number of rates deleted.

        Parameters
        ----------
        rate_ids : typing.Sequence[RateId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        int

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.fee_schedules.v_3.hard_delete_rates_by_ids(
            rate_ids=[
                uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            ],
        )
        """
        _response = self._raw_client.hard_delete_rates_by_ids(rate_ids=rate_ids, request_options=request_options)
        return _response.data


class AsyncV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV3Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV3Client
        """
        return self._raw_client

    async def get_match(
        self, service_line_id: ServiceLineId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[MatchResult]:
        """
        Gets the rate that matches a service line.  No result means no rate exists matching the service line's dimensions.

        Parameters
        ----------
        service_line_id : ServiceLineId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[MatchResult]

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.get_match(
                service_line_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_match(service_line_id, request_options=request_options)
        return _response.data

    async def test_match(
        self,
        service_line_id: ServiceLineId,
        rate_id: RateId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MatchTestResult:
        """
        Tests a service line against a rate to see if it matches.

        Parameters
        ----------
        service_line_id : ServiceLineId

        rate_id : RateId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MatchTestResult

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.test_match(
                service_line_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                rate_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.test_match(service_line_id, rate_id, request_options=request_options)
        return _response.data

    async def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        active_date: typing.Optional[dt.date] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        organization_billing_provider_id: typing.Optional[OrganizationProviderId] = None,
        states: typing.Optional[typing.Union[State, typing.Sequence[State]]] = None,
        zip_codes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        license_types: typing.Optional[typing.Union[LicenseType, typing.Sequence[LicenseType]]] = None,
        facility_type_codes: typing.Optional[typing.Union[FacilityTypeCode, typing.Sequence[FacilityTypeCode]]] = None,
        network_types: typing.Optional[typing.Union[NetworkType, typing.Sequence[NetworkType]]] = None,
        payer_plan_group_ids: typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]] = None,
        cpt_code: typing.Optional[str] = None,
        modifiers: typing.Optional[typing.Union[ProcedureModifier, typing.Sequence[ProcedureModifier]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RatesPage:
        """
        Gets a list of dimensions with their rates. The rates returned will always be the most recent versions of those rates.

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Max number of dimensions returned. Defaults to 100. Max is 100.

        active_date : typing.Optional[dt.date]

        payer_uuid : typing.Optional[PayerUuid]

        organization_billing_provider_id : typing.Optional[OrganizationProviderId]

        states : typing.Optional[typing.Union[State, typing.Sequence[State]]]

        zip_codes : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        license_types : typing.Optional[typing.Union[LicenseType, typing.Sequence[LicenseType]]]

        facility_type_codes : typing.Optional[typing.Union[FacilityTypeCode, typing.Sequence[FacilityTypeCode]]]

        network_types : typing.Optional[typing.Union[NetworkType, typing.Sequence[NetworkType]]]

        payer_plan_group_ids : typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]]

        cpt_code : typing.Optional[str]

        modifiers : typing.Optional[typing.Union[ProcedureModifier, typing.Sequence[ProcedureModifier]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RatesPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.get_multi()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_multi(
            page_token=page_token,
            limit=limit,
            active_date=active_date,
            payer_uuid=payer_uuid,
            organization_billing_provider_id=organization_billing_provider_id,
            states=states,
            zip_codes=zip_codes,
            license_types=license_types,
            facility_type_codes=facility_type_codes,
            network_types=network_types,
            payer_plan_group_ids=payer_plan_group_ids,
            cpt_code=cpt_code,
            modifiers=modifiers,
            request_options=request_options,
        )
        return _response.data

    async def get_unique_values_for_dimension(
        self,
        *,
        pivot_dimension: DimensionName,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        organization_billing_provider_id: typing.Optional[OrganizationProviderId] = None,
        states: typing.Optional[typing.Union[State, typing.Sequence[State]]] = None,
        zip_codes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        license_types: typing.Optional[typing.Union[LicenseType, typing.Sequence[LicenseType]]] = None,
        facility_type_codes: typing.Optional[typing.Union[FacilityTypeCode, typing.Sequence[FacilityTypeCode]]] = None,
        network_types: typing.Optional[typing.Union[NetworkType, typing.Sequence[NetworkType]]] = None,
        payer_plan_group_ids: typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]] = None,
        cpt_code: typing.Optional[str] = None,
        modifiers: typing.Optional[typing.Union[ProcedureModifier, typing.Sequence[ProcedureModifier]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DimensionsPage:
        """
        Gets unique values for a dimension based on other selection criteria. The response is a list of dimensions with your criteria and the unique values populated. This API is useful for driving pivots on dimension values.

        Parameters
        ----------
        pivot_dimension : DimensionName
            The name of the dimension to fetch unique values for.

        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Max number of values returned. Defaults to 1000. Max is 1000.

        payer_uuid : typing.Optional[PayerUuid]

        organization_billing_provider_id : typing.Optional[OrganizationProviderId]

        states : typing.Optional[typing.Union[State, typing.Sequence[State]]]

        zip_codes : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        license_types : typing.Optional[typing.Union[LicenseType, typing.Sequence[LicenseType]]]

        facility_type_codes : typing.Optional[typing.Union[FacilityTypeCode, typing.Sequence[FacilityTypeCode]]]

        network_types : typing.Optional[typing.Union[NetworkType, typing.Sequence[NetworkType]]]

        payer_plan_group_ids : typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]]

        cpt_code : typing.Optional[str]

        modifiers : typing.Optional[typing.Union[ProcedureModifier, typing.Sequence[ProcedureModifier]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DimensionsPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient
        from candid.resources.fee_schedules.resources.v_3 import DimensionName

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.get_unique_values_for_dimension(
                pivot_dimension=DimensionName.PAYER_UUID,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_unique_values_for_dimension(
            pivot_dimension=pivot_dimension,
            page_token=page_token,
            limit=limit,
            payer_uuid=payer_uuid,
            organization_billing_provider_id=organization_billing_provider_id,
            states=states,
            zip_codes=zip_codes,
            license_types=license_types,
            facility_type_codes=facility_type_codes,
            network_types=network_types,
            payer_plan_group_ids=payer_plan_group_ids,
            cpt_code=cpt_code,
            modifiers=modifiers,
            request_options=request_options,
        )
        return _response.data

    async def get_rate_history(
        self, rate_id: RateId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Rate]:
        """
        Gets every version of a rate.

        Parameters
        ----------
        rate_id : RateId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Rate]

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.get_rate_history(
                rate_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_rate_history(rate_id, request_options=request_options)
        return _response.data

    async def upload_fee_schedule(
        self,
        *,
        dry_run: bool,
        rates: typing.Sequence[RateUpload],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Rate]:
        """
        Uploads a new fee schedule.

        Each rate may either be totally new as qualified by it's dimensions or a new version for an existing rate.

        If adding a new version to an existing rate, the rate must be posted with the next version number (previous version + 1) or a EntityConflictError will
        be returned.

        Use the dry run flag to discover already existing rates and to run validations.  If validations for any rate fail, no rates will
        be saved to the system.

        Parameters
        ----------
        dry_run : bool

        rates : typing.Sequence[RateUpload]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Rate]

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            NetworkType,
            ProcedureModifier,
            State,
        )
        from candid.resources.fee_schedules.resources.v_3 import (
            Dimensions,
            RateEntry,
            RateUpload_NewRate,
        )
        from candid.resources.organization_providers.resources.v_2 import LicenseType

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.upload_fee_schedule(
                dry_run=True,
                rates=[
                    RateUpload_NewRate(
                        dimensions=Dimensions(
                            payer_uuid=uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                            organization_billing_provider_id=uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                            states=[State.AA],
                            zip_codes={"zip_codes"},
                            license_types=[LicenseType.MD],
                            facility_type_codes=[FacilityTypeCode.PHARMACY],
                            network_types=[NetworkType.PPO],
                            payer_plan_group_ids=[
                                uuid.UUID(
                                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                                )
                            ],
                            cpt_code="cpt_code",
                            modifiers=[ProcedureModifier.AV],
                        ),
                        entries=[
                            RateEntry(
                                start_date=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                rate_cents=1,
                                is_deactivated=True,
                            ),
                            RateEntry(
                                start_date=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                rate_cents=1,
                                is_deactivated=True,
                            ),
                        ],
                    ),
                    RateUpload_NewRate(
                        dimensions=Dimensions(
                            payer_uuid=uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                            organization_billing_provider_id=uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                            states=[State.AA],
                            zip_codes={"zip_codes"},
                            license_types=[LicenseType.MD],
                            facility_type_codes=[FacilityTypeCode.PHARMACY],
                            network_types=[NetworkType.PPO],
                            payer_plan_group_ids=[
                                uuid.UUID(
                                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                                )
                            ],
                            cpt_code="cpt_code",
                            modifiers=[ProcedureModifier.AV],
                        ),
                        entries=[
                            RateEntry(
                                start_date=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                rate_cents=1,
                                is_deactivated=True,
                            ),
                            RateEntry(
                                start_date=datetime.date.fromisoformat(
                                    "2023-01-15",
                                ),
                                rate_cents=1,
                                is_deactivated=True,
                            ),
                        ],
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upload_fee_schedule(
            dry_run=dry_run, rates=rates, request_options=request_options
        )
        return _response.data

    async def delete_rate(
        self, rate_id: RateId, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Soft deletes a rate from the system.  Only the most recent version of a rate can be deleted.

        Parameters
        ----------
        rate_id : RateId

        version : int

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.delete_rate(
                rate_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                version=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_rate(rate_id, version, request_options=request_options)
        return _response.data

    async def get_payer_thresholds_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerThreshold:
        """
        Gets the default payer threshold

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerThreshold

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.get_payer_thresholds_default()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_payer_thresholds_default(request_options=request_options)
        return _response.data

    async def get_payer_thresholds(
        self,
        *,
        payer_uuids: typing.Union[PayerUuid, typing.Sequence[PayerUuid]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayerThresholdsPage:
        """
        Gets a list of payers and thresholds by their uuids

        Parameters
        ----------
        payer_uuids : typing.Union[PayerUuid, typing.Sequence[PayerUuid]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerThresholdsPage

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.get_payer_thresholds(
                payer_uuids=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_payer_thresholds(
            payer_uuids=payer_uuids, request_options=request_options
        )
        return _response.data

    async def set_payer_threshold(
        self, payer_uuid: PayerUuid, *, request: PayerThreshold, request_options: typing.Optional[RequestOptions] = None
    ) -> PayerThreshold:
        """
        Sets the threshold information for a payer

        Parameters
        ----------
        payer_uuid : PayerUuid

        request : PayerThreshold

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayerThreshold

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.fee_schedules.resources.v_3 import PayerThreshold

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.set_payer_threshold(
                payer_uuid=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=PayerThreshold(
                    disable_paid_incorrectly=True,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set_payer_threshold(
            payer_uuid, request=request, request_options=request_options
        )
        return _response.data

    async def hard_delete_rates(
        self, *, request: OptionalDimensions, request_options: typing.Optional[RequestOptions] = None
    ) -> int:
        """
        Hard deletes rates from the system that match the provided dimensions.  This is a destructive operation and cannot be undone.  If an empty dimensions object is provided, all rates will be hard deleted.  The maximum number of rates this API will delete at a time is 10000.  Returns the number of rates deleted and if that number is the maximum, the caller should call this API again to continue deleting rates.

        Parameters
        ----------
        request : OptionalDimensions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        int

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            NetworkType,
            ProcedureModifier,
            State,
        )
        from candid.resources.fee_schedules.resources.v_3 import OptionalDimensions
        from candid.resources.organization_providers.resources.v_2 import LicenseType

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.hard_delete_rates(
                request=OptionalDimensions(
                    states=[State.AA],
                    zip_codes={"zip_codes"},
                    license_types=[LicenseType.MD],
                    facility_type_codes=[FacilityTypeCode.PHARMACY],
                    network_types=[NetworkType.PPO],
                    payer_plan_group_ids=[
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        )
                    ],
                    modifiers=[ProcedureModifier.AV],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.hard_delete_rates(request=request, request_options=request_options)
        return _response.data

    async def hard_delete_rates_by_ids(
        self, *, rate_ids: typing.Sequence[RateId], request_options: typing.Optional[RequestOptions] = None
    ) -> int:
        """
        Hard deletes specific rates from the system by their IDs. This is a destructive operation and cannot be undone. Limited to 100 rate IDs maximum per request. For bulk deletion of more than 100 rates, use the hard_delete_rates endpoint with dimension filters. Returns the number of rates deleted.

        Parameters
        ----------
        rate_ids : typing.Sequence[RateId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        int

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.fee_schedules.v_3.hard_delete_rates_by_ids(
                rate_ids=[
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.hard_delete_rates_by_ids(rate_ids=rate_ids, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UserConfiguredPromptsResult(UniversalBaseModel):
    """
    Individual result from a single user-configured prompt execution
    """

    user_prompt_id: str = pydantic.Field()
    """
    Reference to the user prompt in eligibility config
    """

    prompt_name: str = pydantic.Field()
    """
    User-defined name for the prompt
    """

    structured_response: typing.Any = pydantic.Field()
    """
    The AI's structured answer (boolean, number, or string)
    """

    exposition: str = pydantic.Field()
    """
    The AI's explanation of the answer
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import httpx
from .core.api_error import ApiError
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.oauth_token_provider import AsyncOAuthTokenProvider, OAuthTokenProvider
from .environment import CandidApiClientEnvironment

if typing.TYPE_CHECKING:
    from .resources.auth.client import AsyncAuthClient, AuthClient
    from .resources.billing_notes.client import AsyncBillingNotesClient, BillingNotesClient
    from .resources.charge_capture.client import AsyncChargeCaptureClient, ChargeCaptureClient
    from .resources.charge_capture_bundles.client import AsyncChargeCaptureBundlesClient, ChargeCaptureBundlesClient
    from .resources.contracts.client import AsyncContractsClient, ContractsClient
    from .resources.credentialing.client import AsyncCredentialingClient, CredentialingClient
    from .resources.custom_schemas.client import AsyncCustomSchemasClient, CustomSchemasClient
    from .resources.diagnoses.client import AsyncDiagnosesClient, DiagnosesClient
    from .resources.eligibility.client import AsyncEligibilityClient, EligibilityClient
    from .resources.encounter_attachments.client import AsyncEncounterAttachmentsClient, EncounterAttachmentsClient
    from .resources.encounter_providers.client import AsyncEncounterProvidersClient, EncounterProvidersClient
    from .resources.encounter_supplemental_information.client import (
        AsyncEncounterSupplementalInformationClient,
        EncounterSupplementalInformationClient,
    )
    from .resources.encounters.client import AsyncEncountersClient, EncountersClient
    from .resources.events.client import AsyncEventsClient, EventsClient
    from .resources.exports.client import AsyncExportsClient, ExportsClient
    from .resources.external_payment_account_config.client import (
        AsyncExternalPaymentAccountConfigClient,
        ExternalPaymentAccountConfigClient,
    )
    from .resources.fee_schedules.client import AsyncFeeSchedulesClient, FeeSchedulesClient
    from .resources.guarantor.client import AsyncGuarantorClient, GuarantorClient
    from .resources.health_care_code_information.client import (
        AsyncHealthCareCodeInformationClient,
        HealthCareCodeInformationClient,
    )
    from .resources.import_invoice.client import AsyncImportInvoiceClient, ImportInvoiceClient
    from .resources.insurance_adjudications.client import (
        AsyncInsuranceAdjudicationsClient,
        InsuranceAdjudicationsClient,
    )
    from .resources.insurance_refunds.client import AsyncInsuranceRefundsClient, InsuranceRefundsClient
    from .resources.medication_dispense.client import AsyncMedicationDispenseClient, MedicationDispenseClient
    from .resources.non_insurance_payer_payments.client import (
        AsyncNonInsurancePayerPaymentsClient,
        NonInsurancePayerPaymentsClient,
    )
    from .resources.non_insurance_payer_refunds.client import (
        AsyncNonInsurancePayerRefundsClient,
        NonInsurancePayerRefundsClient,
    )
    from .resources.non_insurance_payers.client import AsyncNonInsurancePayersClient, NonInsurancePayersClient
    from .resources.organization_providers.client import AsyncOrganizationProvidersClient, OrganizationProvidersClient
    from .resources.organization_service_facilities.client import (
        AsyncOrganizationServiceFacilitiesClient,
        OrganizationServiceFacilitiesClient,
    )
    from .resources.patient_ar.client import AsyncPatientArClient, PatientArClient
    from .resources.patient_payments.client import AsyncPatientPaymentsClient, PatientPaymentsClient
    from .resources.patient_refunds.client import AsyncPatientRefundsClient, PatientRefundsClient
    from .resources.payer_plan_groups.client import AsyncPayerPlanGroupsClient, PayerPlanGroupsClient
    from .resources.payers.client import AsyncPayersClient, PayersClient
    from .resources.pre_encounter.client import AsyncPreEncounterClient, PreEncounterClient
    from .resources.service_lines.client import AsyncServiceLinesClient, ServiceLinesClient
    from .resources.superbills.client import AsyncSuperbillsClient, SuperbillsClient
    from .resources.tasks.client import AsyncTasksClient, TasksClient
    from .resources.write_offs.client import AsyncWriteOffsClient, WriteOffsClient


class CandidApiClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------

    client_id : str
        The client identifier used for authentication.

    client_secret : str
        The client secret used for authentication.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    # or ...

    token : typing.Callable[[], str]
        Authenticate by providing a callable that returns a pre-generated bearer token. In this mode, OAuth client credentials are not required.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from candid import CandidApiClient

    client = CandidApiClient(
        client_id="YOUR_CLIENT_ID",
        client_secret="YOUR_CLIENT_SECRET",
    )

    # or ...

    from candid import CandidApiClient

    client = CandidApiClient(
        base_url="https://yourhost.com/path/to/api",
        token="YOUR_BEARER_TOKEN",
    )
    """

    @typing.overload
    def __init__(
        self,
        *,
        environment: CandidApiClientEnvironment = CandidApiClientEnvironment.PRODUCTION,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        client_id: str,
        client_secret: str,
    ): ...
    @typing.overload
    def __init__(
        self,
        *,
        environment: CandidApiClientEnvironment = CandidApiClientEnvironment.PRODUCTION,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        token: typing.Callable[[], str],
    ): ...
    def __init__(
        self,
        *,
        environment: CandidApiClientEnvironment = CandidApiClientEnvironment.PRODUCTION,
        client_id: typing.Optional[str] = None,
        client_secret: typing.Optional[str] = None,
        token: typing.Optional[typing.Callable[[], str]] = None,
        _token_getter_override: typing.Optional[typing.Callable[[], str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        if token is not None:
            self._client_wrapper = SyncClientWrapper(
                environment=environment,
                httpx_client=httpx_client
                if httpx_client is not None
                else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.Client(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
                token=_token_getter_override if _token_getter_override is not None else token,
            )
        elif client_id is not None and client_secret is not None:
            oauth_token_provider = OAuthTokenProvider(
                client_id=client_id,
                client_secret=client_secret,
                client_wrapper=SyncClientWrapper(
                    environment=environment,
                    httpx_client=httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                    if follow_redirects is not None
                    else httpx.Client(timeout=_defaulted_timeout),
                    timeout=_defaulted_timeout,
                ),
            )
            self._client_wrapper = SyncClientWrapper(
                environment=environment,
                token=_token_getter_override if _token_getter_override is not None else oauth_token_provider.get_token,
                httpx_client=httpx_client
                if httpx_client is not None
                else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.Client(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
            )
        else:
            raise ApiError(
                body="The client must be instantiated with either 'token' or both 'client_id' and 'client_secret'"
            )
        self._auth: typing.Optional[AuthClient] = None
        self._billing_notes: typing.Optional[BillingNotesClient] = None
        self._charge_capture_bundles: typing.Optional[ChargeCaptureBundlesClient] = None
        self._charge_capture: typing.Optional[ChargeCaptureClient] = None
        self._contracts: typing.Optional[ContractsClient] = None
        self._credentialing: typing.Optional[CredentialingClient] = None
        self._custom_schemas: typing.Optional[CustomSchemasClient] = None
        self._eligibility: typing.Optional[EligibilityClient] = None
        self._encounter_attachments: typing.Optional[EncounterAttachmentsClient] = None
        self._encounter_providers: typing.Optional[EncounterProvidersClient] = None
        self._encounter_supplemental_information: typing.Optional[EncounterSupplementalInformationClient] = None
        self._encounters: typing.Optional[EncountersClient] = None
        self._events: typing.Optional[EventsClient] = None
        self._exports: typing.Optional[ExportsClient] = None
        self._external_payment_account_config: typing.Optional[ExternalPaymentAccountConfigClient] = None
        self._fee_schedules: typing.Optional[FeeSchedulesClient] = None
        self._guarantor: typing.Optional[GuarantorClient] = None
        self._health_care_code_information: typing.Optional[HealthCareCodeInformationClient] = None
        self._import_invoice: typing.Optional[ImportInvoiceClient] = None
        self._insurance_adjudications: typing.Optional[InsuranceAdjudicationsClient] = None
        self._insurance_refunds: typing.Optional[InsuranceRefundsClient] = None
        self._medication_dispense: typing.Optional[MedicationDispenseClient] = None
        self._non_insurance_payer_payments: typing.Optional[NonInsurancePayerPaymentsClient] = None
        self._non_insurance_payer_refunds: typing.Optional[NonInsurancePayerRefundsClient] = None
        self._non_insurance_payers: typing.Optional[NonInsurancePayersClient] = None
        self._organization_providers: typing.Optional[OrganizationProvidersClient] = None
        self._organization_service_facilities: typing.Optional[OrganizationServiceFacilitiesClient] = None
        self._patient_ar: typing.Optional[PatientArClient] = None
        self._patient_payments: typing.Optional[PatientPaymentsClient] = None
        self._patient_refunds: typing.Optional[PatientRefundsClient] = None
        self._payer_plan_groups: typing.Optional[PayerPlanGroupsClient] = None
        self._payers: typing.Optional[PayersClient] = None
        self._service_lines: typing.Optional[ServiceLinesClient] = None
        self._superbills: typing.Optional[SuperbillsClient] = None
        self._tasks: typing.Optional[TasksClient] = None
        self._write_offs: typing.Optional[WriteOffsClient] = None
        self._pre_encounter: typing.Optional[PreEncounterClient] = None
        self._diagnoses: typing.Optional[DiagnosesClient] = None

    @property
    def auth(self):
        if self._auth is None:
            from .resources.auth.client import AuthClient  # noqa: E402

            self._auth = AuthClient(client_wrapper=self._client_wrapper)
        return self._auth

    @property
    def billing_notes(self):
        if self._billing_notes is None:
            from .resources.billing_notes.client import BillingNotesClient  # noqa: E402

            self._billing_notes = BillingNotesClient(client_wrapper=self._client_wrapper)
        return self._billing_notes

    @property
    def charge_capture_bundles(self):
        if self._charge_capture_bundles is None:
            from .resources.charge_capture_bundles.client import ChargeCaptureBundlesClient  # noqa: E402

            self._charge_capture_bundles = ChargeCaptureBundlesClient(client_wrapper=self._client_wrapper)
        return self._charge_capture_bundles

    @property
    def charge_capture(self):
        if self._charge_capture is None:
            from .resources.charge_capture.client import ChargeCaptureClient  # noqa: E402

            self._charge_capture = ChargeCaptureClient(client_wrapper=self._client_wrapper)
        return self._charge_capture

    @property
    def contracts(self):
        if self._contracts is None:
            from .resources.contracts.client import ContractsClient  # noqa: E402

            self._contracts = ContractsClient(client_wrapper=self._client_wrapper)
        return self._contracts

    @property
    def credentialing(self):
        if self._credentialing is None:
            from .resources.credentialing.client import CredentialingClient  # noqa: E402

            self._credentialing = CredentialingClient(client_wrapper=self._client_wrapper)
        return self._credentialing

    @property
    def custom_schemas(self):
        if self._custom_schemas is None:
            from .resources.custom_schemas.client import CustomSchemasClient  # noqa: E402

            self._custom_schemas = CustomSchemasClient(client_wrapper=self._client_wrapper)
        return self._custom_schemas

    @property
    def eligibility(self):
        if self._eligibility is None:
            from .resources.eligibility.client import EligibilityClient  # noqa: E402

            self._eligibility = EligibilityClient(client_wrapper=self._client_wrapper)
        return self._eligibility

    @property
    def encounter_attachments(self):
        if self._encounter_attachments is None:
            from .resources.encounter_attachments.client import EncounterAttachmentsClient  # noqa: E402

            self._encounter_attachments = EncounterAttachmentsClient(client_wrapper=self._client_wrapper)
        return self._encounter_attachments

    @property
    def encounter_providers(self):
        if self._encounter_providers is None:
            from .resources.encounter_providers.client import EncounterProvidersClient  # noqa: E402

            self._encounter_providers = EncounterProvidersClient(client_wrapper=self._client_wrapper)
        return self._encounter_providers

    @property
    def encounter_supplemental_information(self):
        if self._encounter_supplemental_information is None:
            from .resources.encounter_supplemental_information.client import (
                EncounterSupplementalInformationClient,  # noqa: E402
            )

            self._encounter_supplemental_information = EncounterSupplementalInformationClient(
                client_wrapper=self._client_wrapper
            )
        return self._encounter_supplemental_information

    @property
    def encounters(self):
        if self._encounters is None:
            from .resources.encounters.client import EncountersClient  # noqa: E402

            self._encounters = EncountersClient(client_wrapper=self._client_wrapper)
        return self._encounters

    @property
    def events(self):
        if self._events is None:
            from .resources.events.client import EventsClient  # noqa: E402

            self._events = EventsClient(client_wrapper=self._client_wrapper)
        return self._events

    @property
    def exports(self):
        if self._exports is None:
            from .resources.exports.client import ExportsClient  # noqa: E402

            self._exports = ExportsClient(client_wrapper=self._client_wrapper)
        return self._exports

    @property
    def external_payment_account_config(self):
        if self._external_payment_account_config is None:
            from .resources.external_payment_account_config.client import (
                ExternalPaymentAccountConfigClient,  # noqa: E402
            )

            self._external_payment_account_config = ExternalPaymentAccountConfigClient(
                client_wrapper=self._client_wrapper
            )
        return self._external_payment_account_config

    @property
    def fee_schedules(self):
        if self._fee_schedules is None:
            from .resources.fee_schedules.client import FeeSchedulesClient  # noqa: E402

            self._fee_schedules = FeeSchedulesClient(client_wrapper=self._client_wrapper)
        return self._fee_schedules

    @property
    def guarantor(self):
        if self._guarantor is None:
            from .resources.guarantor.client import GuarantorClient  # noqa: E402

            self._guarantor = GuarantorClient(client_wrapper=self._client_wrapper)
        return self._guarantor

    @property
    def health_care_code_information(self):
        if self._health_care_code_information is None:
            from .resources.health_care_code_information.client import HealthCareCodeInformationClient  # noqa: E402

            self._health_care_code_information = HealthCareCodeInformationClient(client_wrapper=self._client_wrapper)
        return self._health_care_code_information

    @property
    def import_invoice(self):
        if self._import_invoice is None:
            from .resources.import_invoice.client import ImportInvoiceClient  # noqa: E402

            self._import_invoice = ImportInvoiceClient(client_wrapper=self._client_wrapper)
        return self._import_invoice

    @property
    def insurance_adjudications(self):
        if self._insurance_adjudications is None:
            from .resources.insurance_adjudications.client import InsuranceAdjudicationsClient  # noqa: E402

            self._insurance_adjudications = InsuranceAdjudicationsClient(client_wrapper=self._client_wrapper)
        return self._insurance_adjudications

    @property
    def insurance_refunds(self):
        if self._insurance_refunds is None:
            from .resources.insurance_refunds.client import InsuranceRefundsClient  # noqa: E402

            self._insurance_refunds = InsuranceRefundsClient(client_wrapper=self._client_wrapper)
        return self._insurance_refunds

    @property
    def medication_dispense(self):
        if self._medication_dispense is None:
            from .resources.medication_dispense.client import MedicationDispenseClient  # noqa: E402

            self._medication_dispense = MedicationDispenseClient(client_wrapper=self._client_wrapper)
        return self._medication_dispense

    @property
    def non_insurance_payer_payments(self):
        if self._non_insurance_payer_payments is None:
            from .resources.non_insurance_payer_payments.client import NonInsurancePayerPaymentsClient  # noqa: E402

            self._non_insurance_payer_payments = NonInsurancePayerPaymentsClient(client_wrapper=self._client_wrapper)
        return self._non_insurance_payer_payments

    @property
    def non_insurance_payer_refunds(self):
        if self._non_insurance_payer_refunds is None:
            from .resources.non_insurance_payer_refunds.client import NonInsurancePayerRefundsClient  # noqa: E402

            self._non_insurance_payer_refunds = NonInsurancePayerRefundsClient(client_wrapper=self._client_wrapper)
        return self._non_insurance_payer_refunds

    @property
    def non_insurance_payers(self):
        if self._non_insurance_payers is None:
            from .resources.non_insurance_payers.client import NonInsurancePayersClient  # noqa: E402

            self._non_insurance_payers = NonInsurancePayersClient(client_wrapper=self._client_wrapper)
        return self._non_insurance_payers

    @property
    def organization_providers(self):
        if self._organization_providers is None:
            from .resources.organization_providers.client import OrganizationProvidersClient  # noqa: E402

            self._organization_providers = OrganizationProvidersClient(client_wrapper=self._client_wrapper)
        return self._organization_providers

    @property
    def organization_service_facilities(self):
        if self._organization_service_facilities is None:
            from .resources.organization_service_facilities.client import (
                OrganizationServiceFacilitiesClient,  # noqa: E402
            )

            self._organization_service_facilities = OrganizationServiceFacilitiesClient(
                client_wrapper=self._client_wrapper
            )
        return self._organization_service_facilities

    @property
    def patient_ar(self):
        if self._patient_ar is None:
            from .resources.patient_ar.client import PatientArClient  # noqa: E402

            self._patient_ar = PatientArClient(client_wrapper=self._client_wrapper)
        return self._patient_ar

    @property
    def patient_payments(self):
        if self._patient_payments is None:
            from .resources.patient_payments.client import PatientPaymentsClient  # noqa: E402

            self._patient_payments = PatientPaymentsClient(client_wrapper=self._client_wrapper)
        return self._patient_payments

    @property
    def patient_refunds(self):
        if self._patient_refunds is None:
            from .resources.patient_refunds.client import PatientRefundsClient  # noqa: E402

            self._patient_refunds = PatientRefundsClient(client_wrapper=self._client_wrapper)
        return self._patient_refunds

    @property
    def payer_plan_groups(self):
        if self._payer_plan_groups is None:
            from .resources.payer_plan_groups.client import PayerPlanGroupsClient  # noqa: E402

            self._payer_plan_groups = PayerPlanGroupsClient(client_wrapper=self._client_wrapper)
        return self._payer_plan_groups

    @property
    def payers(self):
        if self._payers is None:
            from .resources.payers.client import PayersClient  # noqa: E402

            self._payers = PayersClient(client_wrapper=self._client_wrapper)
        return self._payers

    @property
    def service_lines(self):
        if self._service_lines is None:
            from .resources.service_lines.client import ServiceLinesClient  # noqa: E402

            self._service_lines = ServiceLinesClient(client_wrapper=self._client_wrapper)
        return self._service_lines

    @property
    def superbills(self):
        if self._superbills is None:
            from .resources.superbills.client import SuperbillsClient  # noqa: E402

            self._superbills = SuperbillsClient(client_wrapper=self._client_wrapper)
        return self._superbills

    @property
    def tasks(self):
        if self._tasks is None:
            from .resources.tasks.client import TasksClient  # noqa: E402

            self._tasks = TasksClient(client_wrapper=self._client_wrapper)
        return self._tasks

    @property
    def write_offs(self):
        if self._write_offs is None:
            from .resources.write_offs.client import WriteOffsClient  # noqa: E402

            self._write_offs = WriteOffsClient(client_wrapper=self._client_wrapper)
        return self._write_offs

    @property
    def pre_encounter(self):
        if self._pre_encounter is None:
            from .resources.pre_encounter.client import PreEncounterClient  # noqa: E402

            self._pre_encounter = PreEncounterClient(client_wrapper=self._client_wrapper)
        return self._pre_encounter

    @property
    def diagnoses(self):
        if self._diagnoses is None:
            from .resources.diagnoses.client import DiagnosesClient  # noqa: E402

            self._diagnoses = DiagnosesClient(client_wrapper=self._client_wrapper)
        return self._diagnoses


class AsyncCandidApiClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------

    client_id : str
        The client identifier used for authentication.

    client_secret : str
        The client secret used for authentication.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    # or ...

    token : typing.Callable[[], str]
        Authenticate by providing a callable that returns a pre-generated bearer token. In this mode, OAuth client credentials are not required.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from candid import AsyncCandidApiClient

    client = AsyncCandidApiClient(
        client_id="YOUR_CLIENT_ID",
        client_secret="YOUR_CLIENT_SECRET",
    )

    # or ...

    from candid import AsyncCandidApiClient

    client = AsyncCandidApiClient(
        base_url="https://yourhost.com/path/to/api",
        token="YOUR_BEARER_TOKEN",
    )
    """

    @typing.overload
    def __init__(
        self,
        *,
        environment: CandidApiClientEnvironment = CandidApiClientEnvironment.PRODUCTION,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        client_id: str,
        client_secret: str,
    ): ...
    @typing.overload
    def __init__(
        self,
        *,
        environment: CandidApiClientEnvironment = CandidApiClientEnvironment.PRODUCTION,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        token: typing.Callable[[], str],
    ): ...
    def __init__(
        self,
        *,
        environment: CandidApiClientEnvironment = CandidApiClientEnvironment.PRODUCTION,
        client_id: typing.Optional[str] = None,
        client_secret: typing.Optional[str] = None,
        token: typing.Optional[typing.Callable[[], str]] = None,
        _token_getter_override: typing.Optional[typing.Callable[[], str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        if token is not None:
            self._client_wrapper = AsyncClientWrapper(
                environment=environment,
                httpx_client=httpx_client
                if httpx_client is not None
                else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.AsyncClient(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
                token=_token_getter_override if _token_getter_override is not None else token,
            )
        elif client_id is not None and client_secret is not None:
            oauth_token_provider = AsyncOAuthTokenProvider(
                client_id=client_id,
                client_secret=client_secret,
                client_wrapper=AsyncClientWrapper(
                    environment=environment,
                    httpx_client=httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                    if follow_redirects is not None
                    else httpx.AsyncClient(timeout=_defaulted_timeout),
                    timeout=_defaulted_timeout,
                ),
            )
            self._client_wrapper = AsyncClientWrapper(
                environment=environment,
                token=_token_getter_override,
                async_token=oauth_token_provider.get_token,
                httpx_client=httpx_client
                if httpx_client is not None
                else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.AsyncClient(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
            )
        else:
            raise ApiError(
                body="The client must be instantiated with either 'token' or both 'client_id' and 'client_secret'"
            )
        self._auth: typing.Optional[AsyncAuthClient] = None
        self._billing_notes: typing.Optional[AsyncBillingNotesClient] = None
        self._charge_capture_bundles: typing.Optional[AsyncChargeCaptureBundlesClient] = None
        self._charge_capture: typing.Optional[AsyncChargeCaptureClient] = None
        self._contracts: typing.Optional[AsyncContractsClient] = None
        self._credentialing: typing.Optional[AsyncCredentialingClient] = None
        self._custom_schemas: typing.Optional[AsyncCustomSchemasClient] = None
        self._eligibility: typing.Optional[AsyncEligibilityClient] = None
        self._encounter_attachments: typing.Optional[AsyncEncounterAttachmentsClient] = None
        self._encounter_providers: typing.Optional[AsyncEncounterProvidersClient] = None
        self._encounter_supplemental_information: typing.Optional[AsyncEncounterSupplementalInformationClient] = None
        self._encounters: typing.Optional[AsyncEncountersClient] = None
        self._events: typing.Optional[AsyncEventsClient] = None
        self._exports: typing.Optional[AsyncExportsClient] = None
        self._external_payment_account_config: typing.Optional[AsyncExternalPaymentAccountConfigClient] = None
        self._fee_schedules: typing.Optional[AsyncFeeSchedulesClient] = None
        self._guarantor: typing.Optional[AsyncGuarantorClient] = None
        self._health_care_code_information: typing.Optional[AsyncHealthCareCodeInformationClient] = None
        self._import_invoice: typing.Optional[AsyncImportInvoiceClient] = None
        self._insurance_adjudications: typing.Optional[AsyncInsuranceAdjudicationsClient] = None
        self._insurance_refunds: typing.Optional[AsyncInsuranceRefundsClient] = None
        self._medication_dispense: typing.Optional[AsyncMedicationDispenseClient] = None
        self._non_insurance_payer_payments: typing.Optional[AsyncNonInsurancePayerPaymentsClient] = None
        self._non_insurance_payer_refunds: typing.Optional[AsyncNonInsurancePayerRefundsClient] = None
        self._non_insurance_payers: typing.Optional[AsyncNonInsurancePayersClient] = None
        self._organization_providers: typing.Optional[AsyncOrganizationProvidersClient] = None
        self._organization_service_facilities: typing.Optional[AsyncOrganizationServiceFacilitiesClient] = None
        self._patient_ar: typing.Optional[AsyncPatientArClient] = None
        self._patient_payments: typing.Optional[AsyncPatientPaymentsClient] = None
        self._patient_refunds: typing.Optional[AsyncPatientRefundsClient] = None
        self._payer_plan_groups: typing.Optional[AsyncPayerPlanGroupsClient] = None
        self._payers: typing.Optional[AsyncPayersClient] = None
        self._service_lines: typing.Optional[AsyncServiceLinesClient] = None
        self._superbills: typing.Optional[AsyncSuperbillsClient] = None
        self._tasks: typing.Optional[AsyncTasksClient] = None
        self._write_offs: typing.Optional[AsyncWriteOffsClient] = None
        self._pre_encounter: typing.Optional[AsyncPreEncounterClient] = None
        self._diagnoses: typing.Optional[AsyncDiagnosesClient] = None

    @property
    def auth(self):
        if self._auth is None:
            from .resources.auth.client import AsyncAuthClient  # noqa: E402

            self._auth = AsyncAuthClient(client_wrapper=self._client_wrapper)
        return self._auth

    @property
    def billing_notes(self):
        if self._billing_notes is None:
            from .resources.billing_notes.client import AsyncBillingNotesClient  # noqa: E402

            self._billing_notes = AsyncBillingNotesClient(client_wrapper=self._client_wrapper)
        return self._billing_notes

    @property
    def charge_capture_bundles(self):
        if self._charge_capture_bundles is None:
            from .resources.charge_capture_bundles.client import AsyncChargeCaptureBundlesClient  # noqa: E402

            self._charge_capture_bundles = AsyncChargeCaptureBundlesClient(client_wrapper=self._client_wrapper)
        return self._charge_capture_bundles

    @property
    def charge_capture(self):
        if self._charge_capture is None:
            from .resources.charge_capture.client import AsyncChargeCaptureClient  # noqa: E402

            self._charge_capture = AsyncChargeCaptureClient(client_wrapper=self._client_wrapper)
        return self._charge_capture

    @property
    def contracts(self):
        if self._contracts is None:
            from .resources.contracts.client import AsyncContractsClient  # noqa: E402

            self._contracts = AsyncContractsClient(client_wrapper=self._client_wrapper)
        return self._contracts

    @property
    def credentialing(self):
        if self._credentialing is None:
            from .resources.credentialing.client import AsyncCredentialingClient  # noqa: E402

            self._credentialing = AsyncCredentialingClient(client_wrapper=self._client_wrapper)
        return self._credentialing

    @property
    def custom_schemas(self):
        if self._custom_schemas is None:
            from .resources.custom_schemas.client import AsyncCustomSchemasClient  # noqa: E402

            self._custom_schemas = AsyncCustomSchemasClient(client_wrapper=self._client_wrapper)
        return self._custom_schemas

    @property
    def eligibility(self):
        if self._eligibility is None:
            from .resources.eligibility.client import AsyncEligibilityClient  # noqa: E402

            self._eligibility = AsyncEligibilityClient(client_wrapper=self._client_wrapper)
        return self._eligibility

    @property
    def encounter_attachments(self):
        if self._encounter_attachments is None:
            from .resources.encounter_attachments.client import AsyncEncounterAttachmentsClient  # noqa: E402

            self._encounter_attachments = AsyncEncounterAttachmentsClient(client_wrapper=self._client_wrapper)
        return self._encounter_attachments

    @property
    def encounter_providers(self):
        if self._encounter_providers is None:
            from .resources.encounter_providers.client import AsyncEncounterProvidersClient  # noqa: E402

            self._encounter_providers = AsyncEncounterProvidersClient(client_wrapper=self._client_wrapper)
        return self._encounter_providers

    @property
    def encounter_supplemental_information(self):
        if self._encounter_supplemental_information is None:
            from .resources.encounter_supplemental_information.client import (
                AsyncEncounterSupplementalInformationClient,  # noqa: E402
            )

            self._encounter_supplemental_information = AsyncEncounterSupplementalInformationClient(
                client_wrapper=self._client_wrapper
            )
        return self._encounter_supplemental_information

    @property
    def encounters(self):
        if self._encounters is None:
            from .resources.encounters.client import AsyncEncountersClient  # noqa: E402

            self._encounters = AsyncEncountersClient(client_wrapper=self._client_wrapper)
        return self._encounters

    @property
    def events(self):
        if self._events is None:
            from .resources.events.client import AsyncEventsClient  # noqa: E402

            self._events = AsyncEventsClient(client_wrapper=self._client_wrapper)
        return self._events

    @property
    def exports(self):
        if self._exports is None:
            from .resources.exports.client import AsyncExportsClient  # noqa: E402

            self._exports = AsyncExportsClient(client_wrapper=self._client_wrapper)
        return self._exports

    @property
    def external_payment_account_config(self):
        if self._external_payment_account_config is None:
            from .resources.external_payment_account_config.client import (
                AsyncExternalPaymentAccountConfigClient,  # noqa: E402
            )

            self._external_payment_account_config = AsyncExternalPaymentAccountConfigClient(
                client_wrapper=self._client_wrapper
            )
        return self._external_payment_account_config

    @property
    def fee_schedules(self):
        if self._fee_schedules is None:
            from .resources.fee_schedules.client import AsyncFeeSchedulesClient  # noqa: E402

            self._fee_schedules = AsyncFeeSchedulesClient(client_wrapper=self._client_wrapper)
        return self._fee_schedules

    @property
    def guarantor(self):
        if self._guarantor is None:
            from .resources.guarantor.client import AsyncGuarantorClient  # noqa: E402

            self._guarantor = AsyncGuarantorClient(client_wrapper=self._client_wrapper)
        return self._guarantor

    @property
    def health_care_code_information(self):
        if self._health_care_code_information is None:
            from .resources.health_care_code_information.client import (
                AsyncHealthCareCodeInformationClient,  # noqa: E402
            )

            self._health_care_code_information = AsyncHealthCareCodeInformationClient(
                client_wrapper=self._client_wrapper
            )
        return self._health_care_code_information

    @property
    def import_invoice(self):
        if self._import_invoice is None:
            from .resources.import_invoice.client import AsyncImportInvoiceClient  # noqa: E402

            self._import_invoice = AsyncImportInvoiceClient(client_wrapper=self._client_wrapper)
        return self._import_invoice

    @property
    def insurance_adjudications(self):
        if self._insurance_adjudications is None:
            from .resources.insurance_adjudications.client import AsyncInsuranceAdjudicationsClient  # noqa: E402

            self._insurance_adjudications = AsyncInsuranceAdjudicationsClient(client_wrapper=self._client_wrapper)
        return self._insurance_adjudications

    @property
    def insurance_refunds(self):
        if self._insurance_refunds is None:
            from .resources.insurance_refunds.client import AsyncInsuranceRefundsClient  # noqa: E402

            self._insurance_refunds = AsyncInsuranceRefundsClient(client_wrapper=self._client_wrapper)
        return self._insurance_refunds

    @property
    def medication_dispense(self):
        if self._medication_dispense is None:
            from .resources.medication_dispense.client import AsyncMedicationDispenseClient  # noqa: E402

            self._medication_dispense = AsyncMedicationDispenseClient(client_wrapper=self._client_wrapper)
        return self._medication_dispense

    @property
    def non_insurance_payer_payments(self):
        if self._non_insurance_payer_payments is None:
            from .resources.non_insurance_payer_payments.client import (
                AsyncNonInsurancePayerPaymentsClient,  # noqa: E402
            )

            self._non_insurance_payer_payments = AsyncNonInsurancePayerPaymentsClient(
                client_wrapper=self._client_wrapper
            )
        return self._non_insurance_payer_payments

    @property
    def non_insurance_payer_refunds(self):
        if self._non_insurance_payer_refunds is None:
            from .resources.non_insurance_payer_refunds.client import AsyncNonInsurancePayerRefundsClient  # noqa: E402

            self._non_insurance_payer_refunds = AsyncNonInsurancePayerRefundsClient(client_wrapper=self._client_wrapper)
        return self._non_insurance_payer_refunds

    @property
    def non_insurance_payers(self):
        if self._non_insurance_payers is None:
            from .resources.non_insurance_payers.client import AsyncNonInsurancePayersClient  # noqa: E402

            self._non_insurance_payers = AsyncNonInsurancePayersClient(client_wrapper=self._client_wrapper)
        return self._non_insurance_payers

    @property
    def organization_providers(self):
        if self._organization_providers is None:
            from .resources.organization_providers.client import AsyncOrganizationProvidersClient  # noqa: E402

            self._organization_providers = AsyncOrganizationProvidersClient(client_wrapper=self._client_wrapper)
        return self._organization_providers

    @property
    def organization_service_facilities(self):
        if self._organization_service_facilities is None:
            from .resources.organization_service_facilities.client import (
                AsyncOrganizationServiceFacilitiesClient,  # noqa: E402
            )

            self._organization_service_facilities = AsyncOrganizationServiceFacilitiesClient(
                client_wrapper=self._client_wrapper
            )
        return self._organization_service_facilities

    @property
    def patient_ar(self):
        if self._patient_ar is None:
            from .resources.patient_ar.client import AsyncPatientArClient  # noqa: E402

            self._patient_ar = AsyncPatientArClient(client_wrapper=self._client_wrapper)
        return self._patient_ar

    @property
    def patient_payments(self):
        if self._patient_payments is None:
            from .resources.patient_payments.client import AsyncPatientPaymentsClient  # noqa: E402

            self._patient_payments = AsyncPatientPaymentsClient(client_wrapper=self._client_wrapper)
        return self._patient_payments

    @property
    def patient_refunds(self):
        if self._patient_refunds is None:
            from .resources.patient_refunds.client import AsyncPatientRefundsClient  # noqa: E402

            self._patient_refunds = AsyncPatientRefundsClient(client_wrapper=self._client_wrapper)
        return self._patient_refunds

    @property
    def payer_plan_groups(self):
        if self._payer_plan_groups is None:
            from .resources.payer_plan_groups.client import AsyncPayerPlanGroupsClient  # noqa: E402

            self._payer_plan_groups = AsyncPayerPlanGroupsClient(client_wrapper=self._client_wrapper)
        return self._payer_plan_groups

    @property
    def payers(self):
        if self._payers is None:
            from .resources.payers.client import AsyncPayersClient  # noqa: E402

            self._payers = AsyncPayersClient(client_wrapper=self._client_wrapper)
        return self._payers

    @property
    def service_lines(self):
        if self._service_lines is None:
            from .resources.service_lines.client import AsyncServiceLinesClient  # noqa: E402

            self._service_lines = AsyncServiceLinesClient(client_wrapper=self._client_wrapper)
        return self._service_lines

    @property
    def superbills(self):
        if self._superbills is None:
            from .resources.superbills.client import AsyncSuperbillsClient  # noqa: E402

            self._superbills = AsyncSuperbillsClient(client_wrapper=self._client_wrapper)
        return self._superbills

    @property
    def tasks(self):
        if self._tasks is None:
            from .resources.tasks.client import AsyncTasksClient  # noqa: E402

            self._tasks = AsyncTasksClient(client_wrapper=self._client_wrapper)
        return self._tasks

    @property
    def write_offs(self):
        if self._write_offs is None:
            from .resources.write_offs.client import AsyncWriteOffsClient  # noqa: E402

            self._write_offs = AsyncWriteOffsClient(client_wrapper=self._client_wrapper)
        return self._write_offs

    @property
    def pre_encounter(self):
        if self._pre_encounter is None:
            from .resources.pre_encounter.client import AsyncPreEncounterClient  # noqa: E402

            self._pre_encounter = AsyncPreEncounterClient(client_wrapper=self._client_wrapper)
        return self._pre_encounter

    @property
    def diagnoses(self):
        if self._diagnoses is None:
            from .resources.diagnoses.client import AsyncDiagnosesClient  # noqa: E402

            self._diagnoses = AsyncDiagnosesClient(client_wrapper=self._client_wrapper)
        return self._diagnoses



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class HumanUserMetadata(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.users.resources.v_2 import HumanUserMetadata

    HumanUserMetadata(
        first_name="John",
        last_name="Doe",
        email="john.doe@example.com",
    )
    """

    first_name: str
    last_name: str
    email: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .denial_reason_content import DenialReasonContent


class ServiceLineDenialReason(UniversalBaseModel):
    """
    The reason a given service line was denied within a given time range.
    A service line may be denied for different reasons over time, but only one reason at a time.
    """

    reason: typing.Optional[DenialReasonContent] = pydantic.Field(default=None)
    """
    Text of the denial reason
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....appointments.resources.v_1.types.appointment import Appointment
from .....appointments.resources.v_1.types.universal_service_identifier import UniversalServiceIdentifier
from .....coverages.resources.v_1.types.mutable_coverage import MutableCoverage
from .....patients.resources.v_1.types.mutable_patient_with_mrn import MutablePatientWithMrn


class AppointmentListItem(UniversalBaseModel):
    appointment: Appointment
    patient: MutablePatientWithMrn
    primary_coverage: typing.Optional[MutableCoverage] = None
    secondary_coverage: typing.Optional[MutableCoverage] = None
    primary_service_type: typing.Optional[UniversalServiceIdentifier] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .health_care_code_information import HealthCareCodeInformation
from .occurrence_span_code import OccurrenceSpanCode
from .occurrence_span_code_qualifier import OccurrenceSpanCodeQualifier
from .rd_8_date import Rd8Date


class OccurrenceSpanInformation(HealthCareCodeInformation):
    """
    This corresponds to the BI code qualifier value.
    """

    occurrence_span_code_qualifier: OccurrenceSpanCodeQualifier
    occurrence_span_code: OccurrenceSpanCode
    occurrence_span_date: Rd8Date

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .state import State


class Regions_States(UniversalBaseModel):
    type: typing.Literal["states"] = "states"
    states: typing.List[State]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Regions_National(UniversalBaseModel):
    type: typing.Literal["national"] = "national"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


Regions = typing_extensions.Annotated[
    typing.Union[Regions_States, Regions_National], pydantic.Field(discriminator="type")
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.date import Date
from .....commons.types.regions import Regions
from ...v_2.types.authorized_signatory import AuthorizedSignatory
from ...v_2.types.contract_status import ContractStatus
from ...v_2.types.insurance_types import InsuranceTypes


class ContractBase(UniversalBaseModel):
    effective_date: Date = pydantic.Field()
    """
    The starting day upon which the contract is effective
    """

    expiration_date: typing.Optional[Date] = pydantic.Field(default=None)
    """
    An optional end day upon which the contract expires
    """

    regions: Regions = pydantic.Field()
    """
    The state(s) to which the contract's coverage extends.
    It may also be set to "national" for the entirety of the US.
    """

    contract_status: typing.Optional[ContractStatus] = None
    authorized_signatory: typing.Optional[AuthorizedSignatory] = None
    commercial_insurance_types: InsuranceTypes = pydantic.Field()
    """
    The commercial plan insurance types this contract applies.
    """

    medicare_insurance_types: InsuranceTypes = pydantic.Field()
    """
    The Medicare plan insurance types this contract applies.
    """

    medicaid_insurance_types: InsuranceTypes = pydantic.Field()
    """
    The Medicaid plan insurance types this contract applies.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .card_payment_method import CardPaymentMethod
    from .card_payment_method_create import CardPaymentMethodCreate
    from .cash_payment_method import CashPaymentMethod
    from .cash_payment_method_create import CashPaymentMethodCreate
    from .check_payment_method import CheckPaymentMethod
    from .check_payment_method_create import CheckPaymentMethodCreate
    from .money_order_payment_method import MoneyOrderPaymentMethod
    from .money_order_payment_method_create import MoneyOrderPaymentMethodCreate
    from .patient_payment import PatientPayment
    from .patient_payment_id import PatientPaymentId
    from .patient_payment_sort_field import PatientPaymentSortField
    from .patient_payments_page import PatientPaymentsPage
    from .payment_method import (
        PaymentMethod,
        PaymentMethod_Card,
        PaymentMethod_Cash,
        PaymentMethod_Check,
        PaymentMethod_MoneyOrder,
    )
    from .payment_method_create import (
        PaymentMethodCreate,
        PaymentMethodCreate_Card,
        PaymentMethodCreate_Cash,
        PaymentMethodCreate_Check,
        PaymentMethodCreate_MoneyOrder,
    )
    from .payment_method_detail import PaymentMethodDetail
    from .payment_method_detail_create import PaymentMethodDetailCreate
_dynamic_imports: typing.Dict[str, str] = {
    "CardPaymentMethod": ".card_payment_method",
    "CardPaymentMethodCreate": ".card_payment_method_create",
    "CashPaymentMethod": ".cash_payment_method",
    "CashPaymentMethodCreate": ".cash_payment_method_create",
    "CheckPaymentMethod": ".check_payment_method",
    "CheckPaymentMethodCreate": ".check_payment_method_create",
    "MoneyOrderPaymentMethod": ".money_order_payment_method",
    "MoneyOrderPaymentMethodCreate": ".money_order_payment_method_create",
    "PatientPayment": ".patient_payment",
    "PatientPaymentId": ".patient_payment_id",
    "PatientPaymentSortField": ".patient_payment_sort_field",
    "PatientPaymentsPage": ".patient_payments_page",
    "PaymentMethod": ".payment_method",
    "PaymentMethodCreate": ".payment_method_create",
    "PaymentMethodCreate_Card": ".payment_method_create",
    "PaymentMethodCreate_Cash": ".payment_method_create",
    "PaymentMethodCreate_Check": ".payment_method_create",
    "PaymentMethodCreate_MoneyOrder": ".payment_method_create",
    "PaymentMethodDetail": ".payment_method_detail",
    "PaymentMethodDetailCreate": ".payment_method_detail_create",
    "PaymentMethod_Card": ".payment_method",
    "PaymentMethod_Cash": ".payment_method",
    "PaymentMethod_Check": ".payment_method",
    "PaymentMethod_MoneyOrder": ".payment_method",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CardPaymentMethod",
    "CardPaymentMethodCreate",
    "CashPaymentMethod",
    "CashPaymentMethodCreate",
    "CheckPaymentMethod",
    "CheckPaymentMethodCreate",
    "MoneyOrderPaymentMethod",
    "MoneyOrderPaymentMethodCreate",
    "PatientPayment",
    "PatientPaymentId",
    "PatientPaymentSortField",
    "PatientPaymentsPage",
    "PaymentMethod",
    "PaymentMethodCreate",
    "PaymentMethodCreate_Card",
    "PaymentMethodCreate_Cash",
    "PaymentMethodCreate_Check",
    "PaymentMethodCreate_MoneyOrder",
    "PaymentMethodDetail",
    "PaymentMethodDetailCreate",
    "PaymentMethod_Card",
    "PaymentMethod_Cash",
    "PaymentMethod_Check",
    "PaymentMethod_MoneyOrder",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.epsdt_referral_condition_indicator_code import EpsdtReferralConditionIndicatorCode


class EpsdtReferralOptional(UniversalBaseModel):
    condition_indicator_1: typing.Optional[EpsdtReferralConditionIndicatorCode] = pydantic.Field(
        alias="condition_indicator1", default=None
    )
    condition_indicator_2: typing.Optional[EpsdtReferralConditionIndicatorCode] = pydantic.Field(
        alias="condition_indicator2", default=None
    )
    condition_indicator_3: typing.Optional[EpsdtReferralConditionIndicatorCode] = pydantic.Field(
        alias="condition_indicator3", default=None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
